function [ts,tu] = mktestset(varargin)
  % @TESTSET/MKTESTSET  Generates the test set from the input data
  %
  % function ts = mktestset(name,desc,tests,callbacks)
  %
  %  This function is the real constructor, this function creates a testset
  % from data isolating the dependence from the order of data in the record.
  %
  %  Params
  %  ------
  % IN:
  %  name         = The string used as name
  %  desc         = A description of the test
  %  tests        =  A cell pf tests, can be testunits or testsets, olso mixed
  %                 (def={})
  %  env          = The initial environment for fixtures (def={})
  %  callbacks    =  The cell of callbacks functions to be called on reset and
  %                 on execution completed. (def={{},{},{},{}})
  % OUT:
  %  ts           = The new test set
  %
  %  Pre
  %  ---
  % -  Name and desc must be strings, tests must be a cell of TestUnits () or
  %   testsets.
  %
  %  Post
  %  ----
  % -  The generated object is a testset... and olso a testunit.
  %
  %  SeeAlso
  %  -------
  % testset, testunit
  %
  %  Examples
  %  --------
  %   This function isn't public.
  
  % Parsing parameters:
  [name,desc,tests,env,callbacks] = ParseParams(varargin{:});
  
  % Generating the TestUnit object
  tu = testunit(@TestingFunction,env,name,desc,callbacks);
  ts.tests = tests;
  
  % ------------------------ LOCAL FUNCTIONS ------------------------
  
  % The testing function
function [tso,success,datao] = TestingFunction(ts,datai)
  
  % Obtaining tests:
  tests = get(ts,'tests');
  
  % Actually all is correct:
  success = 0;
  
  % The output data is the set of data generated from all the sub-tests:
  datao = cell(1,size(tests,2));
  
  % Iterating on the set of sub-tests:
  for ind=1:size(tests,2)
    % Initializing the fixture and gettin the prepared env:
    ts = triggerEvent(ts,3);
    env = get(ts,'data');
    t = tests{ind};
    
    % Preapring the fixture
    if numel(env)>0
      % Obtaining data:
      tdata = get(t,'data');
      % Adding the env:
      tdata2 = {env,tdata{:}};
      % Setting data:
      t = set(t,'data',tdata2);
    end
    
    % Executing the test:
    try
      t = test(t);
      localSuccess = get(t,'status');
      localDatao = get(t,'results');
    catch
      % All bad! (Should never happen)
      err = lasterror;
      
      stck = err.stack;
      for i = 1:length(stck)
        file = stck(i).file;
        disp(['    at <a href="matlab: opentoline(''' file ''',' int2str(stck(i).line) ')">' file ':' int2str(stck(i).line) '</a>']);
      end
      
      
      warning(sprintf(['An error occour in the test: ''%s''\n' ...
        '\tMessage:\t''%s''\n\tIdentifier:\t''%s'''], ...
        get(t,'name'),err.message,err.identifier));
      localSuccess = 1;
      localDatao = {};
    end
    
    % Shutting down the fixture:
    ts = triggerEvent(ts,4);
    
    % Saving the fixture
    if numel(env)>0
      % Saving the old data in the test:
      tests{ind} = set(t,tdata);
    end
    
    % Saving results:
    success = success + localSuccess;
    datao{ind} = {[get(tests{ind},'name')], localDatao};
  end
  
  % Returning:
  tso = set(ts,'tests',tests);
  
  % -----------------------------------------------------------------
  % Parsing of parameters
function [name,desc,tests,env,callbacks] = ParseParams(varargin)
  
  % Minimum number of parameters is 1 (the image)
  if nargin>5 error('Too many parameters'); end
  
  % Set up variables
  varnames = {'name','desc','tests','env','callbacks'};
  for ind=1:nargin
    eval([varnames{ind} ' = varargin{ind} ;']);
  end
  
  % Default parameters
  if nargin<1 name='<noname>'; end
  if nargin<2 desc=''; end
  if nargin<3 tests={}; end
  if nargin<4 env={}; end
  if nargin<5 callbacks={{},{},{},{}}; end
  
  % Check the type and number of elements of callbacks:
  if not(isa(callbacks,'cell'))
    error('Callbacks must be in a cell');
  end
  if size(callbacks,2)<4
    callbacks = {callbacks{:},{},{},{},{}};
    callbacks = callbacks{1:4};
  end
