function tu = addCallback(varargin)
% @TESTUNIT/ADDCALLBACK  Adds a callback to the callback list.
%
%  This function adds a callback to a list of callbacks.
%
%  Params
%  ------
% IN:
%  tu           = The testunit object.
%  func         = The function that represents the callback.
%  type         = The type of callback, can be one of the following values:
%                   1:  Reset callback.
%                   2:  Test completed callback.
%                 the default is 2.
%  head         =  If this parameter is set to true then the callback is
%                 inserted in the head of the list. (def=false)
% OUT:
%  tu           = The new test unit.
%
%  Pre
%  ---
% -  Parameters must be of the type described.
%
%  Post
%  ----
% -  The callback is added (subscribed) ready to be called.
%
%  SeeAlso
%  -------
% testset
%
%  Examples
%  --------
%   Adds a new callback to be executed on test completion:
%  >> tu = addCallback(tu,@myFunc);

% Parsing params:
[tu,func,type,head] = ParseParams(varargin{:});

% Preparing the cell:
cbs = get(tu,'callbacks');
if not(isa(cbs{type},'cell'))
    % Check if the actual content is a single callback:
    if isa(cbs{type},'function_handle')
        % The actual callback must be keeped alive:
        if head cbs{type} = {func,cbs{type}};
        else cbs{type} = {cbs{type},func}; end
    else
        % There aren't other callbacks
        cbs{type} = {func};
    end
else
    % The func can be added
    if head cbs{type} = {func,cbs{type}{:}};
    else cbs{type} = {cbs{type}{:},func}; end
end

% Saving:
tu = set(tu,'callbacks',cbs);

% ------------------------ LOCAL FUNCTIONS ------------------------

% Parsing of parameters
function [tu,func,type,head] = ParseParams(varargin)

% Minimum number of parameters:
if nargin<2 error('Too few parameters'); end
if nargin>4 error('Too many parameters'); end

% Set up variables
varnames = {'tu','func','type','head'};
for ind=1:nargin
    eval([varnames{ind} ' = varargin{ind} ;']);
end

% Default parameters
if nargin<3 type=2; end
if nargin<4 head=false; end

% Check the type
cbs = get(tu,'callbacks');
if size(cbs,2)<type || type<1
    error('The required callback isn''t in the range of known callbacks!');
end
