function tu = mktestunit(varargin)
% @TESTUNIT/MKTESTUNIT  Generates a testunit object
%
% function tu = mktestunit(func,data,name,desc,callbacks)
%
%  This function allow to create a test unit containing all the things that
% can be usefull for the testin and the results analysis.
%
%  Params
%  ------
% IN:
%  func         = The function that represents the test. (def=@identity)
%  data         = Data that are passed to the test as params. (def={})
%  name         = The string used as name. (def='<noname>')
%  desc         = A description of the test. (def='<nodesc>')
%  callbacks    =  The cell of callbacks functions to be called on reset and
%                 on execution completed. (def={{},{}})
% OUT:
%  tu           = The new test unit.
%
%  Pre
%  ---
% -  Name and desc must be strings, func must be a function handle related to 
%   function that recives a test unit and a cell of params and returns two elements:
%    - The success state.
%    - The output data.
%
%  Post
%  ----
% -  The generated object is a testunit.
%
%  SeeAlso
%  -------
% testset, testunit
%
%  Examples
%  --------
%   This function isn't public.

% Parsing parameters:
[func,tdata,name,desc,callbacks] = ParseParams(varargin{:});

% Preparing the output:
tu.func = func;
tu.tdata = tdata;
tu.name = name;
tu.desc = desc;
tu.callbacks = callbacks;
tu.status = false;
tu.stats = [];
tu.results = {};
tu.err = '.';

% ------------------------ LOCAL FUNCTIONS ------------------------

% Parsing of parameters
function [func,tdata,name,desc,callbacks] = ParseParams(varargin)

% Minimum number of parameters is 1 (the image)
if nargin>5 error('Too many parameters'); end

% Set up variables
varnames = {'func','tdata','name','desc','callbacks'};
for ind=1:nargin
    eval([varnames{ind} ' = varargin{ind} ;']);
end

% Default parameters
if nargin<1 func=@identity; end
if nargin<2 tdata={}; end
if nargin<3 name='<noname>'; end
if nargin<4 desc='<nodesc>'; end
if nargin<5 callbacks={{},{}}; end

% Check the type and number of elements of callbacks:
if not(isa(callbacks,'cell'))
    error('Callbacks must be in a cell');
end
if size(callbacks,2)<2
    callbacks = {callbacks{:},{},{}};
    callbacks = {callbacks{1:2}};
end
